@@ -1,4 +1,4 @@
-/* -*- mode: C; c-file-style: "gnu" -*- */
+/* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
 /* dbus-memory.c  D-Bus memory handling
  *
  * Copyright (C) 2002, 2003  Red Hat Inc.
@@ -17,10 +17,11 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 
+#include <config.h>
 #include "dbus-memory.h"
 #include "dbus-internals.h"
 #include "dbus-sysdeps.h"
@@ -105,7 +106,7 @@ static int n_failures_this_failure = 0;
 static dbus_bool_t guards = FALSE;
 static dbus_bool_t disable_mem_pools = FALSE;
 static dbus_bool_t backtrace_on_fail_alloc = FALSE;
-static int n_blocks_outstanding = 0;
+static DBusAtomic n_blocks_outstanding = {0};
 
 /** value stored in guard padding for debugging buffer overrun */
 #define GUARD_VALUE 0xdeadbeef
@@ -232,6 +233,7 @@ _dbus_get_fail_alloc_failures (void)
 }
 
 #ifdef DBUS_BUILD_TESTS
+static dbus_bool_t called = 0;
 /**
  * Called when about to alloc some memory; if
  * it returns #TRUE, then the allocation should
@@ -244,7 +246,17 @@ dbus_bool_t
 _dbus_decrement_fail_alloc_counter (void)
 {
   _dbus_initialize_malloc_debug ();
-  
+#ifdef DBUS_WIN_FIXME
+  {
+    if (!called)
+      {
+        _dbus_verbose("TODO: memory allocation testing errors disabled for now\n");
+        called = 1;
+      }
+    return FALSE;
+  }
+#endif
+
   if (fail_alloc_counter <= 0)
     {
       if (backtrace_on_fail_alloc)
@@ -283,7 +295,7 @@ _dbus_decrement_fail_alloc_counter (void)
 int
 _dbus_get_malloc_blocks_outstanding (void)
 {
-  return n_blocks_outstanding;
+  return _dbus_atomic_get (&n_blocks_outstanding);
 }
 
 /**
@@ -430,6 +442,9 @@ set_guards (void       *real_block,
  * on all platforms. Returns #NULL if the allocation fails.
  * The memory must be released with dbus_free().
  *
+ * dbus_malloc() memory is NOT safe to free with regular free() from
+ * the C library. Free it with dbus_free() only.
+ *
  * @param bytes number of bytes to allocate
  * @return allocated memory, or #NULL if the allocation fails.
  */
@@ -442,11 +457,10 @@ dbus_malloc (size_t bytes)
   if (_dbus_decrement_fail_alloc_counter ())
     {
       _dbus_verbose (" FAILING malloc of %ld bytes\n", (long) bytes);
-      
       return NULL;
     }
 #endif
-  
+
   if (bytes == 0) /* some system mallocs handle this, some don't */
     return NULL;
 #ifdef DBUS_BUILD_TESTS
@@ -458,7 +472,7 @@ dbus_malloc (size_t bytes)
 
       block = malloc (bytes + GUARD_EXTRA_SIZE);
       if (block)
-        n_blocks_outstanding += 1;
+	_dbus_atomic_inc (&n_blocks_outstanding);
       
       return set_guards (block, bytes, SOURCE_MALLOC);
     }
@@ -469,7 +483,7 @@ dbus_malloc (size_t bytes)
       mem = malloc (bytes);
 #ifdef DBUS_BUILD_TESTS
       if (mem)
-        n_blocks_outstanding += 1;
+	_dbus_atomic_inc (&n_blocks_outstanding);
 #endif
       return mem;
     }
@@ -481,6 +495,9 @@ dbus_malloc (size_t bytes)
  * return #NULL if bytes is zero on all platforms. Returns #NULL if the
  * allocation fails.  The memory must be released with dbus_free().
  *
+ * dbus_malloc0() memory is NOT safe to free with regular free() from
+ * the C library. Free it with dbus_free() only.
+ *
  * @param bytes number of bytes to allocate
  * @return allocated memory, or #NULL if the allocation fails.
  */
@@ -497,7 +514,7 @@ dbus_malloc0 (size_t bytes)
       return NULL;
     }
 #endif
-
+  
   if (bytes == 0)
     return NULL;
 #ifdef DBUS_BUILD_TESTS
@@ -509,7 +526,7 @@ dbus_malloc0 (size_t bytes)
 
       block = calloc (bytes + GUARD_EXTRA_SIZE, 1);
       if (block)
-        n_blocks_outstanding += 1;
+	_dbus_atomic_inc (&n_blocks_outstanding);
       return set_guards (block, bytes, SOURCE_MALLOC_ZERO);
     }
 #endif
@@ -519,7 +536,7 @@ dbus_malloc0 (size_t bytes)
       mem = calloc (bytes, 1);
 #ifdef DBUS_BUILD_TESTS
       if (mem)
-        n_blocks_outstanding += 1;
+	_dbus_atomic_inc (&n_blocks_outstanding);
 #endif
       return mem;
     }
@@ -584,7 +601,7 @@ dbus_realloc (void  *memory,
           block = malloc (bytes + GUARD_EXTRA_SIZE);
 
           if (block)
-            n_blocks_outstanding += 1;
+	    _dbus_atomic_inc (&n_blocks_outstanding);
           
           return set_guards (block, bytes, SOURCE_REALLOC_NULL);   
         }
@@ -596,7 +613,7 @@ dbus_realloc (void  *memory,
       mem = realloc (memory, bytes);
 #ifdef DBUS_BUILD_TESTS
       if (memory == NULL && mem != NULL)
-        n_blocks_outstanding += 1;
+	    _dbus_atomic_inc (&n_blocks_outstanding);
 #endif
       return mem;
     }
@@ -617,10 +634,15 @@ dbus_free (void  *memory)
       check_guards (memory, TRUE);
       if (memory)
         {
-          n_blocks_outstanding -= 1;
-          
-          _dbus_assert (n_blocks_outstanding >= 0);
-          
+#ifdef DBUS_DISABLE_ASSERT
+          _dbus_atomic_dec (&n_blocks_outstanding);
+#else
+          dbus_int32_t old_value;
+
+          old_value = _dbus_atomic_dec (&n_blocks_outstanding);
+          _dbus_assert (old_value >= 1);
+#endif
+
           free (((unsigned char*)memory) - GUARD_START_OFFSET);
         }
       
@@ -631,9 +653,14 @@ dbus_free (void  *memory)
   if (memory) /* we guarantee it's safe to free (NULL) */
     {
 #ifdef DBUS_BUILD_TESTS
-      n_blocks_outstanding -= 1;
-      
-      _dbus_assert (n_blocks_outstanding >= 0);
+#ifdef DBUS_DISABLE_ASSERT
+      _dbus_atomic_dec (&n_blocks_outstanding);
+#else
+      dbus_int32_t old_value;
+
+      old_value = _dbus_atomic_dec (&n_blocks_outstanding);
+      _dbus_assert (old_value >= 1);
+#endif
 #endif
 
       free (memory);
@@ -741,16 +768,41 @@ _dbus_register_shutdown_func (DBusShutdownFunction  func,
  */
 
 /**
- * The D-Bus library keeps some internal global variables, for example
- * to cache the username of the current process.  This function is
- * used to free these global variables.  It is really useful only for
- * leak-checking cleanliness and the like. WARNING: this function is
- * NOT thread safe, it must be called while NO other threads are using
- * D-Bus. You cannot continue using D-Bus after calling this function,
- * as it does things like free global mutexes created by
- * dbus_threads_init(). To use a D-Bus function after calling
- * dbus_shutdown(), you have to start over from scratch, e.g. calling
- * dbus_threads_init() again.
+ * Frees all memory allocated internally by libdbus and
+ * reverses the effects of dbus_threads_init(). libdbus keeps internal
+ * global variables, for example caches and thread locks, and it
+ * can be useful to free these internal data structures.
+ *
+ * dbus_shutdown() does NOT free memory that was returned
+ * to the application. It only returns libdbus-internal
+ * data structures.
+ *
+ * You MUST free all memory and release all reference counts
+ * returned to you by libdbus prior to calling dbus_shutdown().
+ *
+ * You can't continue to use any D-Bus objects, such as connections,
+ * that were allocated prior to dbus_shutdown(). You can, however,
+ * start over; call dbus_threads_init() again, create new connections,
+ * and so forth.
+ *
+ * WARNING: dbus_shutdown() is NOT thread safe, it must be called
+ * while NO other threads are using D-Bus. (Remember, you have to free
+ * all D-Bus objects and memory before you call dbus_shutdown(), so no
+ * thread can be using libdbus.)
+ *
+ * The purpose of dbus_shutdown() is to allow applications to get
+ * clean output from memory leak checkers. dbus_shutdown() may also be
+ * useful if you want to dlopen() libdbus instead of linking to it,
+ * and want to be able to unload the library again.
+ *
+ * There is absolutely no requirement to call dbus_shutdown() - in fact,
+ * most applications won't bother and should not feel guilty.
+ * 
+ * You have to know that nobody is using libdbus in your application's
+ * process before you can call dbus_shutdown(). One implication of this
+ * is that calling dbus_shutdown() from a library is almost certainly
+ * wrong, since you don't know what the rest of the app is up to.
+ * 
  */
 void
 dbus_shutdown (void)
